@@ -1,3 +1,51 @@
+
+/***********************************************************************
+ * tcpip.h -- Various functions relating to low level TCP/IP handling, *
+ * including sending raw packets, routing, printing packets, reading   *
+ * from libpcap, etc.                                                  *
+ *                                                                     *
+ ***********************************************************************
+ *  The Nmap Security Scanner is (C) 1995-2001 Insecure.Com LLC. This  *
+ *  program is free software; you can redistribute it and/or modify    *
+ *  it under the terms of the GNU General Public License as published  *
+ *  by the Free Software Foundation; Version 2.  This guarantees your  *
+ *  right to use, modify, and redistribute this software under certain *
+ *  conditions.  If this license is unacceptable to you, we may be     *
+ *  willing to sell alternative licenses (contact sales@insecure.com). *
+ *                                                                     *
+ *  If you received these files with a written license agreement       *
+ *  stating terms other than the (GPL) terms above, then that          *
+ *  alternative license agreement takes precendence over this comment. *
+ *                                                                     *
+ *  Source is provided to this software because we believe users have  *
+ *  a right to know exactly what a program is going to do before they  *
+ *  run it.  This also allows you to audit the software for security   *
+ *  holes (none have been found so far).                               *
+ *                                                                     *
+ *  Source code also allows you to port Nmap to new platforms, fix     *
+ *  bugs, and add new features.  You are highly encouraged to send     *
+ *  your changes to fyodor@insecure.org for possible incorporation     *
+ *  into the main distribution.  By sending these changes to Fyodor or *
+ *  one the insecure.org development mailing lists, it is assumed that *
+ *  you are offering Fyodor the unlimited, non-exclusive right to      *
+ *  reuse, modify, and relicense the code.  This is important because  *
+ *  the inability to relicense code has caused devastating problems    *
+ *  for other Free Software projects (such as KDE and NASM).  Nmap     *
+ *  will always be available Open Source.  If you wish to specify      *
+ *  special license conditions of your contributions, just say so      *
+ *  when you send them.                                                *
+ *                                                                     *
+ *  This program is distributed in the hope that it will be useful,    *
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of     *
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  *
+ *  General Public License for more details (                          *
+ *  http://www.gnu.org/copyleft/gpl.html ).                            *
+ *                                                                     *
+ ***********************************************************************/
+
+/* $Id: tcpip.h,v 1.40 2002/07/20 09:26:25 fyodor Exp $ */
+
+
 #ifndef TCPIP_H
 #define TCPIP_H
 
@@ -5,6 +53,12 @@
 #include "config.h"
 #endif
 
+#include "nbase.h"
+
+#ifdef WIN32
+#include "mswin32\winclude.h"
+#else
+
 #ifdef STDC_HEADERS
 #include <stdlib.h>
 #else
@@ -26,14 +80,6 @@ void *realloc();
 #include <bstring.h>
 #endif
 
-#ifndef HAVE_BZERO
-#define bzero(s, n) memset((s), 0, (n))
-#endif
-
-#ifndef HAVE_MEMCPY
-#define memcpy(d, s, n) bcopy((s), (d), (n))
-#endif
-
 #include <ctype.h>
 #include <sys/types.h>
 
@@ -56,10 +102,23 @@ void *realloc();
 #undef _IP_VHL
 
 #include <stdio.h>
+
+#if HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+
+#ifdef HAVE_RPC_TYPES_H
 #include <rpc/types.h>
+#endif
+
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
+
+#if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+
 #include <arpa/inet.h>
 #ifndef NETINET_IN_SYSTEM_H  /* why the HELL does OpenBSD not do this? */
 #include <netinet/in_systm.h> /* defines n_long needed for netinet/ip.h */
@@ -80,9 +139,12 @@ void *realloc();
 #include <netinet/udp.h>
 #define NETINET_UDP_H
 #endif
+#if HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <fcntl.h>
 #ifndef NET_IF_H  /* why the HELL does OpenBSD not do this? */
+#include <sys/socket.h>
 #include <net/if.h>
 #define NET_IF_H
 #endif
@@ -105,15 +167,25 @@ void *realloc();
 #endif
 
 #include <sys/ioctl.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 #include <pcap.h>
+#ifdef __cplusplus
+}
+#endif
+
 #include <setjmp.h>
 #include <errno.h>
 #include <signal.h>
-#include <pcap.h>
+
 #if HAVE_SYS_SOCKIO_H
 #include <sys/sockio.h>  /* SIOCGIFCONF for Solaris */
 #endif
-#include "error.h"
+#endif /* WIN32 */
+
+#include "nmap_error.h"
 #include "utils.h"
 #include "nmap.h"
 #include "global_structures.h"
@@ -154,7 +226,7 @@ struct interface_info {
 #define HAVE_STRUCT_IP
 
 /* From Linux glibc, which apparently borrowed it from
-   BSD code.  Slightly modified for portability --fyodor@dhp.com */
+   BSD code.  Slightly modified for portability --fyodor@insecure.org */
 /*
  * Structure of an internet header, naked of options.
  */
@@ -291,39 +363,34 @@ int resolve(char *hostname, struct in_addr *ip);
    interface is found, oterwise it returns the device name and fills in the
    source parameter */
 char *routethrough(struct in_addr *dest, struct in_addr *source);
-unsigned short in_cksum(unsigned short *ptr,int nbytes);
-int send_tcp_raw( int sd, struct in_addr *source, 
-		  struct in_addr *victim, unsigned short sport, 
-		  unsigned short dport, unsigned int seq,
-		  unsigned int ack, unsigned char flags,
-		  unsigned short window, char *options, int optlen,
-		  char *data, unsigned short datalen);
-int send_udp_raw( int sd, struct in_addr *source, 
-		  struct in_addr *victim, unsigned short sport, 
-		  unsigned short dport, char *data, unsigned short datalen);
+unsigned short in_cksum(u16 *ptr,int nbytes);
+int send_tcp_raw( int sd, struct in_addr *source, struct in_addr *victim, 
+		  u16 sport, u16 dport, u32 seq, u32 ack, u8 flags,
+		  u16 window, u8 *options, int optlen, char *data, 
+		  u16 datalen);
+int send_udp_raw( int sd, struct in_addr *source, struct in_addr *victim, 
+		  u16 sport, u16 dport, u8 *data, u16 datalen);
+
+int send_ip_raw( int sd, struct in_addr *source, struct in_addr *victim, 
+		 u8 proto, u8 *data, u16 datalen);
+
+/* Much of this is swiped from my send_tcp_raw function above, which 
+   doesn't support fragmentation */
 int send_small_fragz(int sd, struct in_addr *source, struct in_addr *victim,
-                     unsigned long seq, int sport, int dport, int flags);
-int send_ip_raw( int sd, struct in_addr *source, 
-		  struct in_addr *victim, unsigned char proto,
-		  char *data, unsigned short datalen);
-
+		     u32 seq, u16 sport, u16 dport, int flags);
 /* Decoy versions of the raw packet sending functions ... */
-int send_tcp_raw_decoys( int sd, struct in_addr *victim, unsigned short sport, 
-                         unsigned short dport, unsigned int seq,
-                         unsigned int ack, unsigned char flags,
-                         unsigned short window, char *options, int optlen,
-                         char *data, unsigned short datalen);
+int send_tcp_raw_decoys( int sd, struct in_addr *victim, u16 sport, 
+			 u16 dport, u32 seq, u32 ack, u8 flags, u16 window, 
+                         u8 *options, int optlen, u8 *data, u16 datalen);
 
-int send_udp_raw_decoys( int sd, struct in_addr *victim, unsigned short sport, 
-			 unsigned short dport, char *data, 
-			 unsigned short datalen);
+int send_udp_raw_decoys( int sd, struct in_addr *victim, u16 sport, 
+			 u16 dport, u8 *data, u16 datalen);
 
-int send_small_fragz_decoys(int sd, struct in_addr *victim, unsigned long seq, 
-                            int sport, int dport, int flags);
+int send_small_fragz_decoys(int sd, struct in_addr *victim, u32 seq, 
+			    u16 sport, u16 dport, int flags);
 
-
-int send_ip_raw_decoys( int sd, struct in_addr *victim, unsigned char proto,
-                        char *data, unsigned short datalen);
+int send_ip_raw_decoys( int sd, struct in_addr *victim, u8 proto,
+			u8 *data, u16 datalen);
 
 /* Calls pcap_open_live and spits out an error (and quits) if the call fails.
    So a valid pcap_t will always be returned. */
@@ -331,15 +398,15 @@ pcap_t *my_pcap_open_live(char *device, int snaplen, int promisc, int to_ms);
 
 /* A simple function I wrote to help in debugging, shows the important fields
    of a TCP packet*/
-int readtcppacket(char *packet, int readdata);
-int readudppacket(char *packet, int readdata);
+int readtcppacket(unsigned char *packet, int readdata);
+int readudppacket(unsigned char *packet, int readdata);
 /* Convert an IP address to the device (IE ppp0 eth0) using that address */
 int ipaddr2devname( char *dev, struct in_addr *addr );
 /* And vice versa */
 int devname2ipaddr(char *dev, struct in_addr *addr);
 /* Where the above 2 functions get their info */
 struct interface_info *getinterfaces(int *howmany);
-inline void sethdrinclude(int sd);
+void sethdrinclude(int sd);
 int getsourceip(struct in_addr *src, struct in_addr *dst);
 /* Get the source IP and interface name that a packet
    to dst should be sent to.  Interface name is dynamically
@@ -347,7 +414,7 @@ int getsourceip(struct in_addr *src, struct in_addr *dst);
 char *getsourceif(struct in_addr *src, struct in_addr *dst);
 int islocalhost(struct in_addr *addr);
 int unblock_socket(int sd);
-inline int Sendto(char *functionname, int sd, char *packet, int len, 
+int Sendto(char *functionname, int sd, const unsigned char *packet, int len, 
 	   unsigned int flags, struct sockaddr *to, int tolen);
 /* Standard swiped internet checksum routine */
 unsigned short in_cksum(unsigned short *ptr,int nbytes);
@@ -358,6 +425,43 @@ char *readip_pcap(pcap_t *pd, unsigned int *len, long to_usec);
 int inet_aton(register const char *, struct in_addr *);
 #endif
 
+/* Examines the given tcp packet and obtains the TCP timestamp option
+   information if available.  Note that the CALLER must ensure that
+   "tcp" contains a valid header (in particular the th_off must be the
+   true packet length and tcp must contain it).  If a valid timestamp
+   option is found in the header, nonzero is returned and the
+   'timestamp' and 'echots' parameters are filled in with the
+   appropriate value (if non-null).  Otherwise 0 is returned and the
+   parameters (if non-null) are filled with 0.  Remember that the
+   correct way to check for errors is to look at the return value
+   since a zero ts or echots could possibly be valid. */
+int gettcpopt_ts(struct tcphdr *tcp, u32 *timestamp, u32 *echots);
+
+/* Maximize the receive buffer of a socket descriptor (up to 500K) */
+void max_rcvbuf(int sd);
+
+/* Maximize the open file descriptor limit for this process go up to the
+   max allowed  */
+int max_sd();
+
+/* Convert a socket to blocking mode */
+int block_socket(int sd);
+
+/* Give broadcast permission to a socket */
+void broadcast_socket(int sd);
+
+/* Do a receive (recv()) on a socket and stick the results (upt to
+   len) into buf .  Give up after 'seconds'.  Returns the number of
+   bytes read (or -1 in the case of an error.  It only does one recv
+   (it will not keep going until len bytes are read */
+int recvtime(int sd, char *buf, int len, int seconds);
+
+/* This attempts to calculate the round trip time (rtt) to a host by timing a
+   connect() to a port which isn't listening.  A better approach is to time a
+   ping (since it is more likely to get through firewalls (note, this isn't
+   always true nowadays --fyodor).  This is now 
+   implemented in isup() for users who are root.  */
+unsigned long calculate_sleep(struct in_addr target);
 
 #endif /*TCPIP_H*/
 
